/*******************************************************************************
 * Copyright (c) 2007 Genuitec, LLC and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Genuitec, LLC - initial API and implementation
 *******************************************************************************/
package org.eclipse.equinox.prov.reconciler;

import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.equinox.prov.engine.ProvisioningOperand;
import org.eclipse.equinox.prov.metadata.IResolvedInstallableUnit;

public interface ReconcilerChanges // switch to class ?
{
    /**
     * Returns the status of the reconciliation.
     * 
     * @return
     */
    public IStatus getStatus();
    
    /**
     * Returns true if the reconciler found change required in the target profile.
     * 
     * @return
     */
    public boolean hadChanges();
    
    /**
     * Returns the list of software that was uninstalled as a result of the reconciliation.
     * 
     * @return
     */
    public IResolvedInstallableUnit[] getUninstalledSoftware();
    
    /**
     * Returns the list of software that was upgraded as a result of the reconciliation.
     * 
     * @return
     */
    public IResolvedInstallableUnit[] getUpdatedSoftware();
    
    /**
     * Returns the list of software that was installed as a result of the reconciliation.
     * 
     * @return
     */
    public IResolvedInstallableUnit[] getInstalledSoftware();
    
    public void addOperand(ProvisioningOperand operand);

//	public void addOperation(EngineOperation engineOperation);

//	public List getInstallOperations();
//
//	public List getUninstallOperations();
//	
//	public List getUpdateOperations();

    public List getInstallOperands();

    public List getUninstallOperands();

    public List getUpdateOperands();
    
    public void setHadChanges();

	public void setStatus(IStatus status);
}
