package org.eclipse.equinox.prov.reconciler;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.equinox.internal.prov.director.IUTransformationHelper;
import org.eclipse.equinox.prov.core.eventbus.ProvisioningEventBus;
import org.eclipse.equinox.prov.core.eventbus.SynchronousProvisioningListener;
import org.eclipse.equinox.prov.director.IDirector;
import org.eclipse.equinox.prov.engine.InstallOperand;
import org.eclipse.equinox.prov.engine.InstallableUnitEvent;
import org.eclipse.equinox.prov.engine.Profile;
import org.eclipse.equinox.prov.engine.ProvisioningOperand;
import org.eclipse.equinox.prov.engine.UninstallOperand;
import org.eclipse.equinox.prov.engine.UpdateOperand;
import org.eclipse.equinox.prov.metadata.IInstallableUnit;
import org.eclipse.equinox.prov.metadata.InstallableUnit;
import org.eclipse.equinox.prov.metadata.RequiredCapability;
import org.eclipse.osgi.service.resolver.VersionRange;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.Version;

public class SimpleReconciler implements IReconciler {
	
	protected ReconcilerChanges reconcilerChanges;
	
	// FIXME:  implement:
	// - validating target afterwards
	public ReconcilerChanges reconcile(IInstallableUnit mandatory, Profile userProfile,
			IProgressMonitor monitor) {
		
		reconcilerChanges = new SimpleReconcilerChanges();
		InstallableUnit aggregate = new InstallableUnit();
		// FIXME:  better ID
		aggregate.setId("mandatory + user");
		aggregate.setVersion(new Version(0, 0, 0));
		
		ArrayList aggregateIUs = new ArrayList();
		Iterator userIUs = userProfile.getInstallableUnits();
		while (userIUs.hasNext()) {
			IInstallableUnit userIU = (IInstallableUnit) userIUs.next();
			String entryPoint = userIU.getProperty("entryPoint");
			if (entryPoint != null && entryPoint.equalsIgnoreCase("true"))
				aggregateIUs.add(userIU);
		}
		IInstallableUnit[] aggregateIUArray = (IInstallableUnit[]) aggregateIUs.toArray(new IInstallableUnit[aggregateIUs.size()]);
		System.out.println("IUs in aggregateIUArray (just USER):");
		for (int i = 0; i < aggregateIUArray.length; i++) {
			IInstallableUnit installableUnit = aggregateIUArray[i];
			String entryPoint = installableUnit.getProperty("entryPoint");
			if (entryPoint != null && entryPoint.equalsIgnoreCase("true"))
				System.out.println("\t" + installableUnit + " (entryPoint)");
			else
				System.out.println("\t" + installableUnit);
		}
		aggregateIUs.add(mandatory);
		aggregateIUArray = (IInstallableUnit[]) aggregateIUs.toArray(new IInstallableUnit[aggregateIUs.size()]);
		System.out.println("IUs in aggregateIUArray:");
		for (int i = 0; i < aggregateIUArray.length; i++) {
			IInstallableUnit installableUnit = aggregateIUArray[i];
			String entryPoint = installableUnit.getProperty("entryPoint");
			if (entryPoint != null && entryPoint.equalsIgnoreCase("true"))
				System.out.println("\t" + installableUnit + " (entryPoint)");
			else
				System.out.println("\t" + installableUnit);
		}
		aggregate.setRequiredCapabilities(IUTransformationHelper.toRequirements(aggregateIUArray, false));
		
		RequiredCapability[] rcs = aggregate.getRequiredCapabilities();
		for (int i = 0; i < rcs.length; i++) {
			RequiredCapability requiredCapability = rcs[i];
			System.out.println("\taggregateIUs requiredcapability:  " + requiredCapability);
		}
		
//		Iterator targetIUIter = secondary.getInstallableUnits();
//		Collection targetIUs = new ArrayList();
//		while (targetIUIter.hasNext())
//			targetIUs.add((IInstallableUnit) targetIUIter.next());
//		
//		ResolutionHelper sourceStateHelper = new ResolutionHelper(null, null);
////		Collection sourceState = sourceStateHelper.attachCUs(Collections.unmodifiableList(Arrays.asList(mandatoryIUs)));
//		Collection sourceState = sourceStateHelper.attachCUs(Collections.EMPTY_LIST);
//		List sourceStateOrder = sourceStateHelper.getSorted();
//
//		ResolutionHelper targetStateHelper = new ResolutionHelper(secondary.getSelectionContext(), null);
//		Collection targetState = targetStateHelper.attachCUs(targetIUs);
//		List targetStateOrder = targetStateHelper.getSorted();
		
		ServiceReference busReference = ReconcilerActivator.getContext().getServiceReference(ProvisioningEventBus.class.getName());
		ProvisioningEventBus bus = (ProvisioningEventBus) ReconcilerActivator.getContext().getService(busReference);
		
		bus.addListener(new SynchronousProvisioningListener() {
			public void notify(EventObject eventObject) {
				if (eventObject instanceof InstallableUnitEvent) {
					InstallableUnitEvent event = (InstallableUnitEvent) eventObject;
					// If it hasn't happened yet, we don't care.
					if (event.isPre() || !event.getResult().isOK()) {
						return;
					}
					final ProvisioningOperand operand = event.getOperand();
					if (operand instanceof InstallOperand || operand instanceof UninstallOperand || operand instanceof UpdateOperand) {
						reconcilerChanges.addOperand(operand);
						reconcilerChanges.setHadChanges();
					}
				}
			}
		});

		ServiceReference directorReference = ReconcilerActivator.getContext().getServiceReference(IDirector.class.getName());
		IDirector director = (IDirector) ReconcilerActivator.getContext().getService(directorReference);
		
//		EngineOperation[] ops = new OperationGenerator().generateOperation(targetState, sourceState);
//		IStatus applyStatus = new Engine().perform(ops, mandatory, new NullProgressMonitor());
		reconcilerChanges.setStatus(director.become(aggregate, userProfile, monitor));
		return reconcilerChanges;
	}
	
	private IInstallableUnit[] toArray(Iterator it) {
		ArrayList result = new ArrayList();
		while (it.hasNext()) {
			result.add(it.next());
		}
		return (IInstallableUnit[]) result.toArray(new IInstallableUnit[result.size()]);
	}
	
	// FIXME:  remove
	private void printIUs(Profile profile) {
		System.out.println("IUs in " + profile.getProfileId() + ":");
		Iterator ius = profile.getInstallableUnits();
		while (ius.hasNext()) {
			IInstallableUnit iu = (IInstallableUnit) ius.next();
			String entryPoint = iu.getProperty("entryPoint");
			if (entryPoint != null && entryPoint.equalsIgnoreCase("true"))
				System.out.println("\t" + iu + " (entryPoint)");
			else
				System.out.println("\t" + iu);
		}
	}

}
