package org.eclipse.equinox.prov.reconciler;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.equinox.prov.engine.InstallOperand;
import org.eclipse.equinox.prov.engine.ProvisioningOperand;
import org.eclipse.equinox.prov.engine.UninstallOperand;
import org.eclipse.equinox.prov.engine.UpdateOperand;
import org.eclipse.equinox.prov.engine.operations.UpdateOperation;
import org.eclipse.equinox.prov.metadata.IResolvedInstallableUnit;

public class SimpleReconcilerChanges implements ReconcilerChanges {
	
	boolean changes;
	Collection operands;
	IStatus status;
	
	public SimpleReconcilerChanges() {
		operands = new ArrayList();
	}
	
//	public List getInstallOperations() {
//		ArrayList toReturn = new ArrayList();
//		for (Iterator operationIter = operations.iterator(); operationIter.hasNext();) {
//			EngineOperation operation = (EngineOperation) operationIter.next();
//			if (operation instanceof InstallOperation)
//				toReturn.add((InstallOperation) operation);
//		}
//		return toReturn;
//	}

	public IStatus getStatus() {
		return status;
	}
	
	public void setStatus(IStatus status) {
		this.status = status;
	}

//	public List getUninstallOperations() {
//		ArrayList toReturn = new ArrayList();
//		for (Iterator operationIter = operations.iterator(); operationIter.hasNext();) {
//			EngineOperation operation = (EngineOperation) operationIter.next();
//			if (operation instanceof UninstallOperation)
//				toReturn.add((UninstallOperation) operation);
//		}
//		return toReturn;
//	}

	
//	public List getUpdateOperations() {
//		ArrayList toReturn = new ArrayList();
//		for (Iterator operationIter = operations.iterator(); operationIter.hasNext();) {
//			EngineOperation operation = (EngineOperation) operationIter.next();
//			if (operation instanceof UpdateOperation)
//				toReturn.add((UpdateOperation) operation);
//		}
//		return toReturn;
//	}
	
//	public void addOperation(EngineOperation engineOperation) {
//		if (operations == null)
//			operations = new ArrayList();
//		operations.add(engineOperation);
//	}
	
	public void addOperand(ProvisioningOperand operand) {
		operands.add(operand);
	}

	public boolean hadChanges() {
		return changes;
	}
	
	public List getInstallOperands() {
		ArrayList toReturn = new ArrayList();
		for (Iterator operandIter = operands.iterator(); operandIter.hasNext();) {
			ProvisioningOperand operand = (ProvisioningOperand) operandIter.next();
			if (operand instanceof InstallOperand)
				toReturn.add((InstallOperand) operand);
		}
		return toReturn;
	}
	
	public List getUninstallOperands() {
		ArrayList toReturn = new ArrayList();
		for (Iterator operandIter = operands.iterator(); operandIter.hasNext();) {
			ProvisioningOperand operand = (ProvisioningOperand) operandIter.next();
			if (operand instanceof UninstallOperand)
				toReturn.add((UninstallOperand) operand);
		}
		return toReturn;
	}
	
	public List getUpdateOperands() {
		ArrayList toReturn = new ArrayList();
		for (Iterator operandIter = operands.iterator(); operandIter.hasNext();) {
			ProvisioningOperand operand = (ProvisioningOperand) operandIter.next();
			if (operand instanceof UpdateOperand)
				toReturn.add((UpdateOperand) operand);
		}
		return toReturn;
	}

	public IResolvedInstallableUnit[] getInstalledSoftware() {
		IResolvedInstallableUnit[] installedSoftware = new IResolvedInstallableUnit[getInstallOperands().size()];
		int i = 0;
		for (Iterator iterator = getInstallOperands().iterator(); iterator.hasNext();) {
			InstallOperand op = (InstallOperand) iterator.next();
			installedSoftware[i++] = op.getUnitToInstall();
		}
		return installedSoftware;
	}

	public IResolvedInstallableUnit[] getUninstalledSoftware() {
		IResolvedInstallableUnit[] uninstalledSoftware = new IResolvedInstallableUnit[getUninstallOperands().size()];
		int i = 0;
		for (Iterator iterator = getUninstallOperands().iterator(); iterator.hasNext();) {
			UninstallOperand op = (UninstallOperand) iterator.next();
			uninstalledSoftware[i++] = op.getUnitToUninstall();
		}
		return uninstalledSoftware;
	}

	public IResolvedInstallableUnit[] getUpdatedSoftware() {
		IResolvedInstallableUnit[] updatedSoftware = new IResolvedInstallableUnit[getUpdateOperands().size()];
		int i = 0;
		for (Iterator iterator = getUpdateOperands().iterator(); iterator.hasNext();) {
			UpdateOperand op = (UpdateOperand) iterator.next();
			// FIXME:  do we want the original or updated IU?
			updatedSoftware[i++] = op.getUnitToUninstall();
		}
		return updatedSoftware;
	}

	public void setHadChanges() {
		changes = true;
	}
}
