package org.eclipse.equinox.prov.shared.test;

import java.io.File;
import java.util.*;

import junit.framework.TestCase;

import org.eclipse.core.runtime.*;
import org.eclipse.equinox.prov.director.IDirector;
import org.eclipse.equinox.prov.engine.*;
import org.eclipse.equinox.prov.metadata.*;
import org.eclipse.equinox.prov.metadata.repository.IMetadataRepositoryManager;
import org.eclipse.equinox.prov.reconciler.ProfileTemplate;
import org.eclipse.osgi.service.resolver.VersionRange;
import org.osgi.framework.*;

public class AbstractReconcilerTest extends TestCase
{
    protected InstallableUnit newIU(String id, String version)
    {
        InstallableUnit iu = new InstallableUnit();
        iu.setId(id);
        iu.setVersion(new Version(version));
        return iu;
    }
    
    protected RequiredCapability newRequired(String id, String versionRange, boolean optional)
    {
        return RequiredCapability.createRequiredCapabilityForName(id, 
            new VersionRange(versionRange), optional);
    }

    protected Profile newProfile(String name)
    {
        ServiceReference profileRegSr = Activator.context.getServiceReference(IProfileRegistry.class.getName());
        IProfileRegistry profileRegistry = (IProfileRegistry) Activator.context.getService(profileRegSr);

        Profile profile = profileRegistry.getProfile(name);
        assertNull("profile must not exist", profile);
        
        File dir = new File(System.getProperty("java.io.tmpdir"), "provtst-"+name);
        System.out.println("Creating profile ["+dir+"]");
        dir.mkdirs();
        
        profile = new Profile(name);
        profile.setValue(Profile.PROP_INSTALL_FOLDER, dir.getAbsolutePath().replace('\\', '/'));
        
        return profile;
    }

    protected void deleteProfile(Profile profile)
    {
        ServiceReference profileRegSr = Activator.context.getServiceReference(IProfileRegistry.class.getName());
        IProfileRegistry profileRegistry = (IProfileRegistry) Activator.context.getService(profileRegSr);

        profileRegistry.removeProfile(profile);
    }
    
    protected IDirector getDirector()
    {
        ServiceReference directorRegSr = Activator.context.getServiceReference(IDirector.class.getName());
        return (IDirector) Activator.context.getService(directorRegSr);
    }

    protected IInstallableUnit createIUFromTemplate(ProfileTemplate shared)
    {
        InstallableUnit iu = new InstallableUnit();
        iu.setId("shared.profile");
        iu.setVersion(shared.getVersion());
        iu.setRequiredCapabilities(shared.getRequiredCapabilitiesArray());
        return iu;
    }
    
    protected void assertIUs(IInstallableUnit[] ius, Iterator installableUnits)
    {
        Set expected = new HashSet();
        expected.addAll(Arrays.asList(ius));
        
        while (installableUnits.hasNext())
        {
            expected.remove(installableUnits.next());
        }
        
        assertEquals("IUs don't match", 0, expected.size());
    }

    protected void assertNotIUs(IInstallableUnit[] ius, Iterator installableUnits)
    {
        Set notexpected = new HashSet();
        notexpected.addAll(Arrays.asList(ius));
        
        while (installableUnits.hasNext())
        {
            IInstallableUnit next = (IInstallableUnit)installableUnits.next();
            if (notexpected.contains(next))
            {
                fail("not expected ["+next+"]");
            }
        }
    }

    protected IMetadataRepositoryManager getMetadataRepoMgr()
    {
        ServiceReference regsr = Activator.context.getServiceReference(IMetadataRepositoryManager.class.getName());
        IMetadataRepositoryManager metadataMgr =
            (IMetadataRepositoryManager) Activator.context.getService(regsr);
        return metadataMgr;
    }
    
    protected void assertOK(IStatus result) throws CoreException
    {
        if (result.getSeverity() == IStatus.OK)
            return;
        
        Throwable t = result.getException();
        if (t != null)
            t.printStackTrace();
        
        throw new CoreException(result);   
    }
}
