package org.eclipse.equinox.prov.shared.test;

import org.eclipse.core.runtime.*;
import org.eclipse.equinox.prov.engine.Profile;
import org.eclipse.equinox.prov.metadata.*;
import org.eclipse.equinox.prov.metadata.repository.*;
import org.eclipse.equinox.prov.reconciler.ProfileTemplate;
import org.osgi.framework.Version;

public class FirstTimeReconciliation extends AbstractReconcilerTest
{
    public void testFirstTime() throws CoreException
    {
        InstallableUnit iu1 = newIU("test.iu.a", "0.0.1");
        
        InstallableUnit iu2 = newIU("test.iu.b", "0.0.1");
        iu2.setRequiredCapabilities(new RequiredCapability[] {
            newRequired("test.iu.a", "0.0.1", false)
        });
        
        InstallableUnit iu3 = newIU("test.iu.c", "0.0.1");
        iu3.setRequiredCapabilities(new RequiredCapability[] {
            newRequired("test.iu.a", "0.0.1", false),
            newRequired("test.iu.b", "0.0.1", false)
        });
        
        ProfileTemplate shared = new ProfileTemplate(new Version("0.0.1"));
        shared.setRequiredCapabilities(new RequiredCapability[] {
            newRequired("test.iu.a", "0.0.1", false),
            newRequired("test.iu.b", "0.0.1", false),
            newRequired("test.iu.c", "0.0.1", false)
        });
                                
        IMetadataRepositoryManager metadataMgr = getMetadataRepoMgr();
        IInstallableUnit ius[] = new IInstallableUnit[]{ iu1, iu2, iu3 };

        IMetadataRepository metadataRepo = new TestMetadataRepository(ius);
        metadataMgr.addRepository(metadataRepo);
        try
        {
            Profile profile = newProfile("firsttime");
            
            assertOK(getDirector().install(new IInstallableUnit[] {
            
                // For now, we'll do an install as an IU
                createIUFromTemplate(shared)
            
            }, profile, null, new NullProgressMonitor()));
            
            assertIUs(ius, profile.getInstallableUnits());
            
            deleteProfile(profile);
        }
        finally
        {
            metadataMgr.removeRepository(metadataRepo);
        }
    }
}